% /**************************************************************************************************************************
% 
% ---------------------------------------------------------------------------------------------------------------------------
% 
% Purpose:   Estimate resistance based on formulation by Lindqvist 
% 
% Theory:   Lindqvist, G. A straightforward method for calculation of ice resistance 
% of ships. in The 10th International Conference on Port and Ocean Engineering 
% under Arctic Conditions. 1989. Lule. Equation numbers refer to this
% paper
% 
% Called from: AnalytRes.m
% Subroutines: None
% Comments
% 
% 
% Variables:
% Name           I/O       Description
% -------------------------------------------------------------------------------------------------------------------------
% sigma           I	Ice bending strength [Pa]
% drho            I	difference between salt water density and ice density
% phi             I	Ship bow angle
% alpha           I	Water line entrance angle
% T               I	Draught
% B               I	Breath
% Lpp             I	Length between perpendiculars
% H               I	Ice thickness [m]
% v               I	vessel speed [m/s]
% my              I     Friction coefficient between ice and hull  
% lindqv	  O	Estimate of resistance [kN]
% 
% E			Youngs modulus
% ny			poisson ratio
% g		 	Acceleration of gravity [m/s^2]
% rho_w			salt water density [kg/m^3]
% 
% ---------------------------------------------------------------------------------------------------------------------------
% Programmed by: Torstein Skr
% Last changed/ version:050211/1.0
% **************************************************************************************************************************/


function lindqv=lindqvist(sigma,drho,phi,alpha,T,B,Lpp,H,v,my)% angles in degrees, length in SI, resistance in kN
%convert angles to radians
alpha=radians(alpha);
phi=radians(phi);
psi=atan(tan(phi)/sin(alpha));

% constants
E=9e9;%Youngs modulus
ny=.33;%poisson ratio
g=9.81; %[m/s^2] Acceleration of gravity
rho_w=1025;%salt water density


% Chrushing eq(1) and (2)
Fv=.5*sigma*H^2;
Rc=Fv*(tan(phi)+my*cos(phi)/cos(psi))/(1-my*sin(phi)/cos(psi));

% Bending eq(17)
Rb=(27/64)*sigma*B*H^1.5/(sqrt(E/(12*(1-ny^2)*g*rho_w)))*(tan(psi)+my*cos(phi)/(cos(psi)*sin(alpha)))*(1+1/cos(psi));

% Submergence eq(6)
Rs=drho*g*H*B*(T*(B+T)/(B+2*T)+my*(.7*Lpp-T/tan(phi)-B/(4*tan(alpha))+T*cos(phi)*cos(psi)*sqrt(1/(sin(phi)^2)+1/tan(alpha)^2)));

% Speed dependency eq (7)
lindqv=(Rc+Rb)*(1+1.4*v/sqrt(g*H))+Rs*(1+9.4*v/sqrt(g*Lpp));
lindqv=lindqv*1e-3;%[kN]

% keyboard
