% Analyses the ratio between calculated resistance and analytical estimates
% function [label RegOut] = regression (compare,label,RegParam)


global Torstein
%% calculate ratios, and create labels for later use
ratio=zeros(size(compare,1),4);
for i=1:size(compare,1)
    ratio(i,:)=[compare(i,2) compare(i,3) compare(i,4)/compare(i,5) compare(i,4)/(compare(i,6))];
end
label.ratio={label.compare{2}; label.compare{3}; [label.compare{4}(1:3),'/',label.compare{5}(1:6)];[label.compare{4}(1:3),'/',label.compare{6}(1:5)]};
%remove NaNs to prevent error
ratio(any(isnan(ratio),2),:) = [];

%load position vectors and line plot variables
[pos lst]=plotvar;

%% Scatterplot both ratios against both parameters, before sorting
if RegParam.fullScatter==true
    j=1;
    for x=1:2
        for y=3:4
            % data smoothing
            yy=smooth(ratio(:,x),ratio(:,y),31);
            smoothplot=[ratio(:,x) yy];
            smoothplot=sortrows(smoothplot,1);
            % plot
            figure('Name',[label.ratio{y},' ratio plotted against ',label.ratio{x}],...
                'NumberTitle','off','OuterPosition',pos{j})
            hold on
            scatter(ratio(:,x),ratio(:,y),'.')
            plot(smoothplot(:,1),smoothplot(:,2),'-r')
            title([label.ratio{y},' ratio plotted against ',label.ratio{x}])
            ylabel('Ratio')
            legend('Data','Moving average')
            xlabel(label.ratio{x})
            axis([0 4 0 5])
            j=j+1;
        end
    end
end
%% Fit surface to ratios
if RegParam.surffit==true
    [surface.Rfitresult, surface.Rgof] = createSurfaceFitRobust(ratio(:,1), ratio(:,2),ratio(:,4),'Riska');
    fprintf('\n-----------------------------------------------\nRiska robust surface fit:\n'); surface.Rfitresult
    fprintf('Goodness of fit paramters:\n'); disp(surface.Rgof)
    [surface.Lfitresult, surface.Lgof] = createSurfaceFitRobust(ratio(:,1), ratio(:,2),ratio(:,3),'Lindqvist');
    fprintf('-----------------------------------------------\nLindqvist robust surface fit:\n'); surface.Lfitresult
    fprintf('Goodness of fit paramters:\n'); disp(surface.Lgof)
    RegOut.surf=surface;
end

%% Divide measurements in subgroups
S=RegParam.bins(1);T=RegParam.bins(2);%Number of bins in speed and thickness domain

SpeedThres=0:4/S:4;
ThickThres=0:3/T:3;
sort.ratios=cell(S,T);%sorted values
sort.normal=cell(S,T);%normal distr. data, with same structure as in sort.ratios
label.sort=cell(S,T);
sort.logn.Rdist=cell(S,T);
sort.logn.Ldist=cell(S,T);%Lognormal distr data for riska and lindqv ratios, incl confidence interval
sort.logn.Rresidual=cell(S,T);
sort.logn.Lresidual=cell(S,T);

counter=ones(S,T);
% Sorting algorithm
for i=1:size(ratio,1)
    for j=1:S
        if ratio(i,1)>=SpeedThres(j) && ratio(i,1)<SpeedThres(j+1)%If correct speed bin
            for k=1:T
                if ratio(i,2)>=ThickThres(k) && ratio(i,2)<ThickThres(k+1)%If correct thickness bin
                    sort.ratios{j,k}(counter(j,k),:)=ratio(i,:);                    
                    counter(j,k)=counter(j,k)+1;
                end
            end
        end
    end
end

% Create labels
for i=1:S
    for j=1:T
        label.sort{i,j}=['V \in[',num2str(SpeedThres(i),'%4.2g'),',',num2str(SpeedThres(i+1),'%4.2g'),']  h \in[',num2str(ThickThres(j),'%4.2g'),',',num2str(ThickThres(j+1),'%4.2g'),']'];

    end
end

%% Calculate stat. values and test distibutions for subgroups
alpha=.05;
RL={'L','R'};
for i=1:2% both ratios
    for s=1:S% all speeds
        for t=1:T% all thickness
            if size(sort.ratios{s,t},1)>RegParam.MinObs % don't calculate stat. values if to few observations
                % Normal distribution data (mean and standard deviation)
                sort.normal{s,t}(1:2,:)=[mean(sort.ratios{s,t},1);std(sort.ratios{s,t},0,1)];
                % Log-Normal distribution data
                [parmhat parmci]=lognfit(sort.ratios{s,t}(:,i+2),alpha);
                
                %% chi-square test of lognormal distribution, incl error handling
                %disable warning, and reset warning variable
                warning('off','stats:chi2gof:LowCounts')
                lastwarn('','')
                %test of lognormal dist on Riska data
                [h p str]=chi2gof(sort.ratios{s,t}(:,i+2),'cdf',{@logncdf,parmhat(1),parmhat(2)},'nparams',2);
                [WarningString WarningID]=lastwarn;
                chi2error=~isempty(WarningID);
                if chi2error==true
                    disp(' ')
                    disp(['Warning from chi2gof on ',RL{i},' data in cell {',num2str(s),',',num2str(t),'}:'])
                    disp(WarningString)
                end
                lastwarn('','')
                sort.logn.([RL{i},'dist']){s,t}=[h p;parmhat ;parmci;chi2error 0];
                
            end
        end
    end
end
warning('on','stats:chi2gof:LowCounts')
label.lognormal={'Chi-square test result (0=accept lognormal)','Propability of getting data set from lognormal distr';'mu','sigma'...
    ;'mu lower confidence bound','sigma lower conf. bound';'mu upper conf. interval','sigma upper conf. interval';'chi2gof bin error','0'};
%% test of chi2 on wbl distr
% data=sort.ratios{3,4}(:,4);
% p = mle(data,'dist','lognormal');% calc weibull parameters
% fun = @(data,mu,sig)cdf('lognormal',data,mu,sig);% create function handle for cumulative distribution function
% [h_test p_test]=chi2gof(data,'cdf',{fun,p(1),p(2)},'nparams',2);
% [parmhat_r]=lognfit(data,alpha);
% [h_r p_r]=chi2gof(data,'cdf',{@logncdf,parmhat_r(1),parmhat_r(2)},'nparams',2);

%% Plot sorted data in subplots with scatter
if RegParam.dividedScatter==true
    for x=1:2
        for y=3:4
            a=figure('Name',[label.ratio{y},' ratio plotted against ',label.ratio{x}],...
                'NumberTitle','off','OuterPosition',scrsz);
            i=1;
            for s=1:S
                for t=1:T                    
                    h(i)=subplot(S,T,i);                    
                    if isempty(sort.ratios{s,t})==false % Only plot if data
                        scatter(sort.ratios{s,t}(:,x),sort.ratios{s,t}(:,y),'.')
                        axis ([min(sort.ratios{s,t}(:,x)) max(sort.ratios{s,t}(:,x)) 0 2])                       
                        NumTicks = 3;
                        L = get(gca,'XLim');
                        ticks=linspace(L(1),L(2),NumTicks);
                        set(gca,'XTick',ticks)
                        set(gca,'xticklabel',num2str(ticks.','%5.1f'));
                        set(h(i),'YTick',1:2);
                    else                                          
                        text(.1,.5,'{\bf No data}')
                    end
                    i=i+1;
                end                
            end
            %print main title
            mtit([label.ratio{y},' ratio plotted against ',label.ratio{x}],'fontsize',16,'color',[0 0 0],'xoff',0,'yoff',.025)	     
            % Fix labels on main x and y axis
            k=1;
            for j=1:S:i-1
                set(get(h(j),'YLabel'),'String',['V\in[',num2str(SpeedThres(k),'%11.2g'),' , ',num2str(SpeedThres(k+1),'%11.2g'),']'])
                k=k+1;
            end
            k=1;
            for j=S*T-T+1:S*T
                set(get(h(j),'XLabel'),'String',['h\in[',num2str(ThickThres(k),'%11.2g'),' , ',num2str(ThickThres(k+1),'%11.2g'),']'])
                k=k+1;
            end
            if RegParam.save==true
                filename=['Plots\Ratios\',label.ratio{y}(1:3),label.ratio{y}(5:9),' vs ',label.ratio{x}(1:6),'-splitted.tif'];
                saveas(a,filename)
                close
            end
        end
    end
end
%% Plot sorted data in a more structured way, incl moving average
if RegParam.intScatter==true   
for y=3:4;% for both riska and lindqvist ratio
    % Plot against speed
    a=figure('Name',[label.ratio{y},' ratio plotted against ',label.ratio{1}],...
        'NumberTitle','off','OuterPosition',pos{5});
    h=cell(1,T);
    for t =1:T        
        h{t}=subplot(T,1,T-t+1);
        p=get(h{t},'pos');
        p(4)=p(4)+0.02*5/T; %decrease space between scatterplots
        set(h{t},'pos',p)
        xvar=0;yvar=0;
        for s=1:S% collect plotting terms over relevant cells
            xvar=vertcat(xvar,sort.ratios{s,t}(:,1));  yvar=vertcat(yvar,sort.ratios{s,t}(:,y));
        end
        % data smoothing
        yy=smooth(xvar,yvar,31);
        smpl=[xvar yy];
        smpl=sortrows(smpl,1);        
        hold on
        scatter(xvar,yvar,'.')
        plot(smpl(:,1),smpl(:,2),'-r')
        axis([0 4 0 2])
        ylabel({'H [m]\in';['[',num2str(ThickThres(t),'%11.2g'),',',num2str(ThickThres(t+1),'%11.2g'),'] '];'Ratio'})
        set(gca,'xticklabel',[])
        grid on
    end
    %activate xticks on lower subplot
    set(h{1},'xtickMode', 'auto','xticklabelMode','auto')
    set(get(h{1},'XLabel'),'String',label.ratio{1})
    %save file
    if RegParam.save==true
        filename=['Plots\Ratios\',label.ratio{y}(1:4),label.ratio{y}(6:10),' vs ',label.ratio{1}(1:6),'.emf'];
        saveas(a,filename)
        close
    end
    % Plot against thickness
    a=figure('Name',[label.ratio{y},' ratio plotted against ',label.ratio{2}],...
        'NumberTitle','off','OuterPosition',pos{5});
    h=cell(1,S);
    for s =1:S        
        h{s}=subplot(S,1,S-s+1);
        p=get(h{s},'pos');
        p(4)=p(4)+0.02*5/S; %decrease space between scatterplots
        set(h{s},'pos',p)
        xvar=0;yvar=0;
        for t=1:T% collect plotting terms over relevant cells
            xvar=vertcat(xvar,sort.ratios{s,t}(:,2));  yvar=vertcat(yvar,sort.ratios{s,t}(:,y));
        end
        yy=smooth(xvar,yvar,31);
        smpl=[xvar yy];
        smpl=sortrows(smpl,1);        
        hold on
        scatter(xvar,yvar,'.')
        plot(smpl(:,1),smpl(:,2),'-r')
        axis([0 3 0 2])
        ylabel({'V [m/s]\in';['[',num2str(SpeedThres(s),'%11.2g'),',',num2str(SpeedThres(s+1),'%11.2g'),'] '];'Ratio'})
        set(gca,'xticklabel',[])
        grid on
    end    
    %acticate x-tics on lower subplot
    set(h{1},'xtickMode', 'auto','xticklabelMode','auto')
    set(get(h{1},'XLabel'),'String',label.ratio{2})
    %save image
    if RegParam.save==true
        filename=['Plots\Ratios\',label.ratio{y}(1:4),label.ratio{y}(6:10),' vs ',label.ratio{2}(1:12),'.emf'];
        saveas(a,filename)
        close
    end
end
end
%% Plot Log-Normal distribution, incl confidence interval

if RegParam.lognormal(1)==true
    if RegParam.lognormal(3)==true
        subpl=[2,2];
    else
        subpl=[1,2];
    end
    if isfield(RegParam,'distr')==true%additonal distributions
        for i=1:size(RegParam.distr,2)
            sort.(RegParam.distr{i}).Rresidual=cell(S,T);
            sort.(RegParam.distr{i}).Lresidual=cell(S,T);
        end
    end
    for i=1:2% both ratios
        for s=1:S %all speeds
            for t=1:T %all thickness
                if size(sort.ratios{s,t},1)>RegParam.MinObs %only plot if data set is not empty
                    h=figure('Name',[label.ratio{i+2},' ratio tested as a Log-Normal distribution in interval: ',label.sort{s,t},'    s=',num2str(s),' t=',num2str(t)],...
                        'NumberTitle','off','OuterPosition',pos{5});
                    subplot(subpl(1),subpl(2),1)
                    % normalized histogram and lognormal curves
                    histnorm(sort.ratios{s,t}(:,4))
                    hold on
                    % plot Log-NormalProbability density function, incl conf. intervals
                    x=0:.01:max(sort.ratios{s,t}(:,4)+.5);
                    y=lognpdf(x,sort.logn.([RL{i},'dist']){s,t}(2,1),sort.logn.([RL{i},'dist']){s,t}(2,2));%(x,mu,sigma)
                    ymax=lognpdf(x,sort.logn.([RL{i},'dist']){s,t}(4,1),sort.logn.([RL{i},'dist']){s,t}(4,2));
                    ymin=lognpdf(x,sort.logn.([RL{i},'dist']){s,t}(3,1),sort.logn.([RL{i},'dist']){s,t}(3,2));
                    plot(x,y,'r',x,ymin,'m',x,ymax,'g','LineWidth',1.5)
                    %                 title(label.sort{s,t})
                    legend('Normalized histogram','Log-Normal PDF',['Lower ',num2str((1-alpha)*100),'% Confidence interval'],...
                        ['Upper ',num2str((1-alpha)*100),'% Confidence interval'],'Location','SouthOutside')
                    xlabel([label.ratio{i+2},' ratio'])
                    hold off
                    
                    hPrbPl=subplot(subpl(1),subpl(2),2);% probability plots
                    data = sort.ratios{s,t}(:,4);
                    probplot('lognormal',data);% lognormal probability plot
                    sort.logn.([RL{i},'residual']){s,t}=Torstein;
                    legnd={'Log Normal','Data'};
                    if isfield(RegParam,'distr')==true %fit additional distributions
                        for j=1:size(RegParam.distr,2)
                            p = mle(data,'dist',RegParam.distr{j});% calc distribution parameters
                            fun = @(data,mu,sig)cdf(RegParam.distr{j},data,mu,sig);% create function handle for cumulative distribution function
                            ax = probplot(hPrbPl,fun,p);% Plot line of new cdf in prob. plot
                            set(ax,'color',lst.col{j+3},'linestyle','-')
                            sort.(RegParam.distr{j}).([RL{i},'residual']){s,t}=Torstein;
                        end
                        legnd={'Log Normal','Data',RegParam.distr{1:i}};
                    end
                    title(' ')
                    legend(legnd,'Location','SO')
                    mtit(h,[label.ratio{i+2},' ratio tested as a Log-Normal distribution in interval: ',label.sort{s,t}],'fontsize',14,'color',[0 0 0],'xoff',0,'yoff',.025)
                    %Check chi squared test result
                    if sort.logn.([RL{i},'dist']){s,t}(1,1)==0&&sort.logn.([RL{i},'dist']){s,t}(5,1)==0
                        txt={'\chi^2 test performed without error:'; 'Log-Normality accepted';[' P=',num2str(sort.logn.([RL{i},'dist']){s,t}(1,2))]};
                    elseif sort.logn.([RL{i},'dist']){s,t}(1,1)==1&&sort.logn.([RL{i},'dist']){s,t}(5,1)==0
                        txt={'\chi^2 test performed without error:'; 'Log-Normality not accepted';[' P=',num2str(sort.logn.([RL{i},'dist']){s,t}(1,2))]};
                    else
                        txt={'error in \chi^2 test.','See command window for furter info'};
                    end
                    annotation(h,'textbox',[0.58 0.81 .2 .1],...
                        'String',txt,...
                        'BackgroundColor',[1 1 1],'FitBoxToText','on');
                    if RegParam.lognormal(3)==true % Scatterplott
                        subplot(subpl(1),subpl(2),3)
                        scatter(sort.ratios{s,t}(:,1),sort.ratios{s,t}(:,i+2),'.')
                        title([label.ratio{i+2},' ratio plotted against ',label.ratio{1}])
                        xlabel(label.ratio{1})
                        ylabel('ratio')
                        % Scatterplot
                        subplot(subpl(1),subpl(2),4)
                        scatter(sort.ratios{s,t}(:,2),sort.ratios{s,t}(:,i+2),'.')
                        title([label.ratio{i+2},' ratio plotted against ',label.ratio{2}])
                        xlabel(label.ratio{2})
                        ylabel('ratio')
                    end
                    if RegParam.save==true
                        filename=['Plots\Lognormal distribution\',label.ratio{i+2}(1:3),label.ratio{i+2}(5:9),...
                            ' V =[',num2str(SpeedThres(s),'%11.2g'),',',num2str(SpeedThres(s+1),'%11.2g'),'] h=[',num2str(ThickThres(t),'%11.2g'),...
                            ',',num2str(ThickThres(t+1),'%11.2g'),'].tif'];
                        saveas(h,filename)
                        close
                    end
                    
                    % needs modified probplot. to work:
                    % line 80: insert global torstein;
                    % line 178: insert torstein.linefun=linefun;
                    % line 278: insert global torstein; torstein.ydata=q; torstein.xdata=x;
                    
                end
                %             break
                
            end
            %         break
        end
        %% Plot residuals from main distribution
        if RegParam.lognormal(2)==true
            SSR=cell(4,1);
            name=['Residuals from statistical fitting of ',label.ratio{i+2},' ratio to Log-Normal Distribution'];
            figure('Name',name,'NumberTitle','off','OuterPosition',pos{5});
            residuals{i}=sort.logn.([RL{i},'residual']);
            k=1;clear legnd;
            for s=1:S
                for t=1:T
                    if isempty(residuals{i}{s,t})==false %only attempt to plot if data
                        [hLine SSR{i+2}(s,t)]=probresiduals(residuals{i}{s,t},1);%line handle and average squared error
                        set(hLine,'Color',lst.col{s},'LineStyle',lst.style{t})%,'Marker',lst.marker{t}
                        legnd{k}=label.sort{s,t};
                        k=k+1;
                    end
                end
            end
            axis tight
            xaxis=get(gca, 'XLim');
            plot(xaxis,[0 0],'r','LineWidth',1)
            ylim([-3 3])
            legend(legnd,'Location','EO')
            xlabel('Data','FontSize',14)
            ylabel('Relative distance from distribution line','FontSize',14)
            title(name, 'FontSize',14)
            
            if RegParam.save==true
                filename=['Plots\Lognormal distribution\',label.ratio{i+2}(5:length(label.ratio{i+2})),'residuals.tif'];
                saveas(gca,filename)
            end
            %% residuals for sec. prob distr
            if isfield(RegParam,'distr')==true% check for multiple distributions
                for j=1:size(RegParam.distr,2)
                    name=['Residuals from statistical fitting of ',label.ratio{i+2},' ratio to ',RegParam.distr{j},'  Distribution'];
                    figure('Name',name,'NumberTitle','off','OuterPosition',pos{5});
                    k=1;
                    for s=1:S
                        for t=1:T
                            if isempty(sort.(RegParam.distr{j}).([RL{i},'residual']){s,t})==false %only attempt to plot if data
                                [hLine SSR{y}(s,t)]=probresiduals(sort.(RegParam.distr{j}).([RL{i},'residual']){s,t},2);%line handle and average squared error
                                set(hLine,'Color',lst.col{s},'LineStyle',lst.style{t})%,'Marker',lst.marker{t}
                                legnd{k}=label.sort{s,t};
                                k=k+1;
                            end
                        end
                    end
                    axis tight
                    xaxis=get(gca, 'XLim');
                    plot(xaxis,[0 0],'r','LineWidth',1)
                    ylim([-3 3])
                    legend(legnd,'Location','EO')
                    xlabel('Data','FontSize',14)
                    ylabel('Relative distance from distribution line','FontSize',14)
                    title(name, 'FontSize',14)
                end
                if RegParam.save==true
                    filename=['Plots\Lognormal distribution\',label.ratio{i+2}(5:length(label.ratio{i+2})),'residuals.tif'];
                    saveas(gca,filename)
                end
            end
            
            
        end
    end
end



%% save output
RegOut.sort=sort;


%% Documentation
% Possible string values for distribution selection using "mle" and "cdf":
% 'kernel' 
% 'beta'
% 'binomial'
% 'birnbaumsaunders'
% 'exponential'
% 'extreme value' 
% 'gamma'
% 'generalized extreme value' 
% 'generalized pareto' 
% 'inversegaussian'
% 'logistic'
% 'loglogistic'
% 'lognormal'
% 'nakagami'
% 'negative binomial' 
% 'normal'
% 'poisson'
% 'rayleigh'
% 'rician'
% 'tlocationscale'
% 'weibull'

% Possible string values for distribution selection using "probplot":
% 'exponential'      Exponential probability plot (nonnegative values)
% 'extreme value'    Extreme value probability plot (all values)
% 'lognormal'        Lognormal probability plot (positive values)
% 'normal'           Normal probability plot (all values)
% 'rayleigh'         Rayleigh probability plot (positive values)
% 'weibull'          Weibull probability plot (positive values)